import pygame
from pgcenteredbutton.colors import *
from pgcenteredbutton.offset_linear_regression import params

def left_click(event: pygame.event.Event):
    return event.type == pygame.MOUSEBUTTONDOWN and event.button == 1

class Button:
    def __init__(self, screen: pygame.Surface, text: str, COLOR: tuple, center: tuple, dim: tuple, thickness = 1, radius = -1, font_size = 0):
        if font_size == 0:
            self.font_size = int(dim[1]*1/.9) 
        else:
            self.font_size = font_size
        font = pygame.font.Font(None, self.font_size)
        self.render = font.render(text, True, COLOR)
        
        self.screen = screen
        self.screen_color = screen.get_at(center)

        self.COLOR = COLOR
        self.brightened = False
        self.was_hovered = False
        self.clicked = False

        self.radius = radius
        self.thickness = thickness
        self.dim = dim # (w, h)
        self.real_rect = pygame.Rect(0, 0, self.dim[0], self.dim[1]) #(left, top, width, height)
        self.real_rect.center = center

        self.font_rect = self.render.get_rect(center=center) #for centering text

        while self.font_rect.w > dim[0] or self.font_rect.h > dim[1]: #manual rect is too small
            self.font_size -= 1
            font = pygame.font.Font(None, self.font_size)
            self.render = font.render(text, True, COLOR)
            self.font_rect = self.render.get_rect(center=center)

        self.font_rect.centery = self.font_rect.centery+self.font_size*params['coef']+params['intercept'] #move text to center

        
    def get_text_bottom(self):
        #bottom of real rect - empty space between bottom and text. font_size/2 is the actual pixel height of the text
         return int(self.real_rect.bottom-(self.real_rect.h-self.font_size//2+1)//2)
    
    def get_text_top(self):
        return int(self.real_rect.top+(self.real_rect.h-self.font_size//2+1)//2)

    def draw_text(self):
        self.screen.blit(self.render, self.font_rect)

    def draw_border(self, extra = 0, color = None):
        if color == None:
            color = self.COLOR
        pygame.draw.rect(self.screen, color, self.real_rect, self.thickness + extra, self.radius)

    def draw(self):
        self.draw_text()
        self.draw_border()

    def draw_hovered(self, color = WHITE):
        self.draw_border(3, color)

    def erase_button(self):
        self.draw_border(-self.thickness, self.screen_color)

    def erase_hovered(self):
        self.draw_hovered(self.screen_color)
        self.draw_border()
    
    def draw_clicked(self):
        brighter_color = tuple([value+(255-value)*0.2 for value in self.screen_color.normalize()[:3]])
        self.draw_border(-self.thickness, brighter_color)
        self.draw_text()
        self.draw_hovered()

    def is_hovered(self):
        return self.real_rect.collidepoint(pygame.mouse.get_pos())

    #dont need event to check, but if there is one, check if its mouseclick
    def check_clicked(self):
        if self.clicked:
            self.clicked = False
            return True
        if self.is_hovered():
            self.draw_hovered()
            pygame.display.update()
            while self.is_hovered():        
                pygame.mouse.set_cursor(pygame.SYSTEM_CURSOR_HAND)
                for event in pygame.event.get():
                    if left_click(event):   
                        self.draw_clicked()
                        pygame.display.update()
                        self.brightened = True
                        while self.is_hovered():  
                            for new_event in pygame.event.get():
                                if new_event.type == pygame.MOUSEBUTTONUP and event.button == 1:
                                    pygame.mouse.set_cursor(pygame.SYSTEM_CURSOR_ARROW)
                                    self.erase_button()
                                    self.screen.blit(self.render, self.font_rect)
                                    self.draw_hovered()
                                    pygame.display.update()
                                    self.clicked = True
                                    return True
        
        pygame.mouse.set_cursor(pygame.SYSTEM_CURSOR_ARROW)
        if self.brightened:
            self.brightened = False
            self.erase_button()
            self.draw()
        else:
            self.erase_hovered()
        pygame.display.update()
        return False
        




import numpy as np
from sklearn.linear_model import LinearRegression
from pgcenteredbutton.generate_offset_data import generate_offset_data
import json


try:
    with open("offset_data.json", "r") as file:
        dict = json.load(file)
except:
    generate_offset_data()
    with open("offset_data.json", "r") as file:
        dict = json.load(file)

x_data = dict['font_size_list']
y_data = dict['offset_list']

# Given data
x_data = np.array(x_data)  # Continue with all x values
y_data = np.array(y_data)  # Continue with all y values

# Reshape the data for scikit-learn
x_data = x_data.reshape(-1, 1)
y_data = y_data.reshape(-1, 1)

# Create and train the linear regression model
model = LinearRegression()
model.fit(x_data, y_data)


# Calculate the absolute percentage error for each data point
ape = np.abs(y_data / x_data) * 100

# Calculate MAPE
mape = np.mean(ape)

params = {'coef': float(model.coef_[0][0]), 'intercept': float(model.intercept_[0]), 'MAPE': mape}

try:
    with open('data_analysis.json', 'r') as file:
        data = json.load(file)
except:
    data = {}

data['unadjusted_data'] = params

with open('data_analysis.json', 'w') as file:
    json.dump(data, file, indent=4)


import pygame
import json
from pgcenteredbutton.colors import *

def generate_offset_data():
    clock = pygame.time.Clock()


    pygame.init()
    dim = (1000,600)
    screen = pygame.display.set_mode((dim[0],dim[1]))
    screen.fill((BLACK))

    real_rect = pygame.rect.Rect((0,0), dim)
    real_rect.center = screen.get_rect().center

    COLOR = GREEN

    font_list = []
    offset_list = []
    for font_size in range(5,800):
        
        screen.fill(BLACK)
        
        font = pygame.font.Font(None, font_size)
        render = font.render('B', True, COLOR)
        font_rect = render.get_rect(center = real_rect.center)
        screen.blit(render, font_rect)
        pygame.draw.rect(screen, COLOR, real_rect, 1)

        ##initial value
        text_top = -1 
        text_bot = -1

        rect_centery = dim[1]//2+1
        
        #0 and height-1 are the rectangle border
        i = 0
        while text_top < 0 or text_bot < 0:
            
            for y in range(1,dim[1]//2):
                pos = (dim[0]//2+i,y)
                color = screen.get_at(pos)[1]

                #get center of text and and size of text
                #offset -> move down the difference in pixels between text center and rectangle center for each font_size
                if color > 0:
                    text_top = y

                    break

            for y in range(dim[1]-2,dim[1]//2,-1):
                pos = (dim[0]//2+i,y)
                color = screen.get_at(pos)[1]

                #get center of text and and size of text
                #offset -> move down the difference in pixels between text center and rectangle center for each font_size
                if color > 0:
                    text_bot = y

                    break
            
            i = i+1

        letter_height = text_bot-text_top+1
        letter_centery = text_top + letter_height//2 + 1

        offset = rect_centery - letter_centery

        font_list.append(font_size)
        offset_list.append(offset)

        pygame.display.update()

    with open("offset_data.json", "w") as file:
        json.dump({'font_size_list': font_list, 'offset_list': offset_list}, file)

    pygame.quit()
    exit()




from setuptools import setup, find_packages

setup(
    name='pygame-truly-centered-button',
    version='1.1',
    packages=find_packages(),
    install_requires=[
        'pygame', 'numpy', 'joblib', 'scipy', 'threadpoolctl', 'scikit-learn'
    ],
    author='Melvin Chen',
    author_email='melvinchen610@gmail.com',
    description='A Pygame button library that fixes text-centering',
    long_description=open('README.md').read(),
    long_description_content_type='text/markdown',
    url='https://github.com/mchen610/pygame-truly-centered-button',
)


import numpy as np
from sklearn.linear_model import LinearRegression
from pgcenteredbutton.generate_adjusted_offset_data import generate_adjusted_offset_data
import json

def generate_adjusted_linear_regression():
    try:
        with open("offset_data_adjusted.json", "r") as file:
            dict = json.load(file)
    except:
        generate_adjusted_offset_data()
        with open("offset_data_adjusted.json", "r") as file:
            dict = json.load(file)
            
    x_data = dict['font_size_list']
    y_data = dict['offset_list']

    x_data = np.array(x_data)
    y_data = np.array(y_data)

    x_data = x_data.reshape(-1, 1)
    y_data = y_data.reshape(-1, 1)

    model = LinearRegression()
    model.fit(x_data, y_data)


    ape = np.abs(y_data/x_data) * 100

    mape = np.mean(ape)

    params = {'coef': float(model.coef_[0][0]), 'intercept': float(model.intercept_[0]), 'MAPE': mape}
    print(params)
    try:
        with open('data_analysis.json', 'r') as file:
            data = json.load(file)
    except:
        data = {}

    data['adjusted_data'] = params

    with open('data_analysis.json', 'w') as file:
        json.dump(data, file, indent=4)
